@@ -126,6 +126,12 @@ void *uu_FileCBArg = NULL;
 void *uu_FFCBArg   = NULL;
 void *uu_FNCBArg;
 
+int uu_autocheck = 1;           /* call UUCheckGlobaList after every part */
+
+/* I/O buffer sizes */
+int uu_rbuf = 0;
+int uu_wbuf = 0;
+
 /*
  * Global variables
  */
@@ -236,7 +242,7 @@ static allomap toallocate[] = {
   { &uuestr_itemp,       256 },  /* from uuencode.c:UUEncodeStream() */
   { &uuestr_otemp,      1024 },
   { &uulib_msgstring,   1024 },  /* from uulib.c:UUMessage() */
-  { &uuncdl_fulline,    1200 },  /* from uunconc.c:UUDecodeLine() */
+  { &uuncdl_fulline,    1201 },  /* from uunconc.c:UUDecodeLine() */
   { &uuncdp_oline,      3600 },  /* from uunconc.c:UUDecodePart() */
   { &uunconc_UUxlat,     256 * sizeof (int) },  /* from uunconc.c:toplevel */
   { &uunconc_UUxlen,      64 * sizeof (int) },
@@ -248,10 +254,10 @@ static allomap toallocate[] = {
   { &uuscan_shlline2,   1024 },  /* from uuscan.c:ScanHeaderLine() */
   { &uuscan_pvvalue,     300 },  /* from uuscan.c:ParseValue() */
   { &uuscan_phtext,      300 },  /* from uuscan.c:ParseHeader() */
-  { &uuscan_sdline,      300 },  /* from uuscan.c:ScanData() */
+  { &uuscan_sdline,     1025 },  /* from uuscan.c:ScanData(), +1 for UURepairData */
   { &uuscan_sdbhds1,     300 },
   { &uuscan_sdbhds2,     300 },
-  { &uuscan_spline,      300 },  /* from uuscan.c:ScanPart() */
+  { &uuscan_spline,     1024 },  /* from uuscan.c:ScanPart() */
   { &uuutil_bhwtmp,      300 },  /* from uuutil.c:UUbhwrite() */
   { NULL, 0 }
 };
@@ -269,7 +275,7 @@ UUMessage (va_alist)
   va_dcl
 #endif
 {
-  char *msgptr;
+  int msgofs;
 #if defined(STDC_HEADERS) || defined(HAVE_STDARG_H)
   va_list ap;
 
@@ -287,16 +293,16 @@ UUMessage (va_alist)
 #endif
 
   if (uu_debug) {
-    sprintf (uulib_msgstring, "%s(%d): %s", file, line, msgnames[level]);
-    msgptr = uulib_msgstring + strlen (uulib_msgstring);
+    snprintf (uulib_msgstring, 1024,  "%s(%d): %s", file, line, msgnames[level]);
+    msgofs = strlen (uulib_msgstring);
   }
   else {
-    sprintf (uulib_msgstring, "%s", msgnames[level]);
-    msgptr = uulib_msgstring + strlen (uulib_msgstring);
+    snprintf (uulib_msgstring, 1024, "%s", msgnames[level]);
+    msgofs = strlen (uulib_msgstring);
   }
 
   if (uu_MsgCallback && (level>UUMSG_NOTE || uu_verbose)) {
-    vsprintf (msgptr, format, ap);
+    vsnprintf (uulib_msgstring + msgofs, 1024 - msgofs, format, ap);
 
     (*uu_MsgCallback) (uu_MsgCBArg, uulib_msgstring, level);
   }
@@ -421,6 +427,14 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
   int result;
 
   switch (option) {
+  case UUOPT_RBUF:
+    *ivalue = uu_rbuf;
+    result = 0;
+    break;
+  case UUOPT_WBUF:
+    *ivalue = uu_wbuf;
+    result = 0;
+    break;
   case UUOPT_VERSION:
     _FP_strncpy (cvalue, uulibversion, clength);
     result = 0;
@@ -505,6 +519,10 @@ UUGetOption (int option, int *ivalue, char *cvalue, int clength)
     if (ivalue) *ivalue = uu_dotdot;
     result = uu_dotdot;
     break;
+  case UUOPT_AUTOCHECK:
+    if (ivalue) *ivalue = uu_autocheck;
+    result = uu_autocheck;
+    break;
   default:
     return -1;
   }
@@ -515,6 +533,12 @@ int UUEXPORT
 UUSetOption (int option, int ivalue, char *cvalue)
 {
   switch (option) {
+  case UUOPT_RBUF:
+    uu_rbuf = ivalue;
+    break;
+  case UUOPT_WBUF:
+    uu_wbuf = ivalue;
+    break;
   case UUOPT_FAST:
     uu_fast_scanning  = ivalue;
     break;
@@ -568,6 +592,9 @@ UUSetOption (int option, int ivalue, char *cvalue)
   case UUOPT_DOTDOT:
     uu_dotdot = ivalue;
     break;
+  case UUOPT_AUTOCHECK:
+    uu_autocheck = ivalue;
+    break;
   default:
     return UURET_ILLVAL;
   }
@@ -682,12 +709,13 @@ UULoadFile (char *filename, char *fileid, int delflag, int *partcount)
 int UUEXPORT
 UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int *partcount)
 {
-  int res, sr, count=0;
+  int res, sr;
   struct stat finfo;
   fileread *loaded;
   uufile *fload;
   itbd *killem;
   FILE *datei;
+  void *datei_buf;
 
   int _count;
   if (!partcount)
@@ -701,12 +729,14 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
 	       filename, strerror (uu_errno = errno));
     return UURET_IOERR;
   }
+  UUSETBUF (datei, datei_buf, uu_rbuf);
 
   if (fstat (fileno(datei), &finfo) == -1) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_NOT_STAT_FILE),
 	       filename, strerror (uu_errno = errno));
     fclose (datei);
+    UUCLRBUF (uu_rbuf, datei_buf);
     return UURET_IOERR;
   }
 
@@ -749,7 +779,7 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
     /* 
      * Peek file, or some systems won't detect EOF
      */
-    res = fgetc (datei);
+    res = _FP_fgetc (datei);
     if (feof (datei) || ferror (datei))
       break;
     else
@@ -763,9 +793,10 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
 		     uustring (S_READ_ERROR), filename,
 		     strerror (uu_errno));
 
-	UUCheckGlobalList ();
+	if (uu_autocheck) UUCheckGlobalList ();
 	progress.action = 0;
 	fclose (datei);
+        UUCLRBUF (uu_rbuf, datei_buf);
 	return sr;
       }
       continue;
@@ -775,9 +806,10 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_READ_ERROR), filename,
 		 strerror (uu_errno = errno));
-      UUCheckGlobalList ();
+      if (uu_autocheck) UUCheckGlobalList ();
       progress.action = 0;
       fclose (datei);
+      UUCLRBUF (uu_rbuf, datei_buf);
       return UURET_IOERR;
     }
 
@@ -843,9 +875,10 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
       UUkillfile (fload);
 
       if (res != UURET_NODATA) {
-	UUCheckGlobalList ();
+	if (uu_autocheck) UUCheckGlobalList ();
 	progress.action = 0;
 	fclose (datei);
+        UUCLRBUF (uu_rbuf, datei_buf);
 	return res;
       }
       if (uu_fast_scanning && sr != UURET_CONT)
@@ -865,23 +898,27 @@ UULoadFileWithPartNo (char *filename, char *fileid, int delflag, int partno, int
     if (uu_fast_scanning && sr != UURET_CONT)
       break;
   }
+
   if (ferror (datei)) {
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_READ_ERROR), filename,
 	       strerror (uu_errno = errno));
-    UUCheckGlobalList ();
+    if (uu_autocheck) UUCheckGlobalList ();
     progress.action = 0;
     fclose (datei);
+    UUCLRBUF (uu_rbuf, datei_buf);
     return UURET_IOERR;
   }
+
   fclose (datei);
+  UUCLRBUF (uu_rbuf, datei_buf);
 
   if (!uu_fast_scanning && *partcount == 0)
     UUMessage (uulib_id, __LINE__, UUMSG_NOTE,
 	       uustring (S_NO_DATA_FOUND), filename);
 
   progress.action = 0;
-  UUCheckGlobalList ();
+  if (uu_autocheck) UUCheckGlobalList ();
 
   return UURET_OK;
 }
@@ -904,6 +941,7 @@ int UUEXPORT
 UUDecodeFile (uulist *thefile, char *destname)
 {
   FILE *target, *source;
+  void *target_buf, *source_buf;
   struct stat finfo;
   int fildes, res;
   size_t bytes;
@@ -927,6 +965,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       thefile->binfile, strerror (uu_errno = errno));
     return UURET_IOERR;
   }
+  UUSETBUF (source, source_buf, uu_rbuf);
 
   /*
    * for system security, strip setuid/setgid bits from mode
@@ -953,10 +992,10 @@ UUDecodeFile (uulist *thefile, char *destname)
   if (destname)
     strcpy (uugen_fnbuffer, destname);
   else {
+    char *fname = UUFNameFilter (thefile->filename ? thefile->filename : "unknown.xxx");
     sprintf (uugen_fnbuffer, "%.1024s%.3071s",
-	     (uusavepath)?uusavepath:"",
-	     UUFNameFilter ((thefile->filename)?
-			    thefile->filename:"unknown.xxx"));
+	     uusavepath ? uusavepath : "",
+             fname ? fname : "unknown.xxx");
   }
 
   /*
@@ -968,6 +1007,7 @@ UUDecodeFile (uulist *thefile, char *destname)
       UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_TARGET_EXISTS), uugen_fnbuffer);
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       return UURET_EXISTS;
     }
   }
@@ -977,6 +1017,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_NOT_STAT_FILE),
 	       thefile->binfile, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     return UURET_IOERR;
   }
 
@@ -988,6 +1029,7 @@ UUDecodeFile (uulist *thefile, char *destname)
       mask = umask (0022); umask (mask);
 #endif
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
 #if HAVE_CHMOD
       chmod (uugen_fnbuffer, thefile->mode & ~mask);
 #endif
@@ -1014,6 +1056,7 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_NOT_OPEN_TARGET),
 	       uugen_fnbuffer, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     return UURET_IOERR;
   }
 
@@ -1023,9 +1066,11 @@ UUDecodeFile (uulist *thefile, char *destname)
 	       uustring (S_IO_ERR_TARGET),
 	       uugen_fnbuffer, strerror (uu_errno = errno));
     fclose (source);
+    UUCLRBUF (uu_rbuf, source_buf);
     close  (fildes);
     return UURET_IOERR;
   }
+  UUSETBUF (target, target_buf, uu_wbuf);
 
   while (!feof (source)) {
 
@@ -1033,7 +1078,9 @@ UUDecodeFile (uulist *thefile, char *destname)
       UUMessage (uulib_id, __LINE__, UUMSG_NOTE,
 		 uustring (S_DECODE_CANCEL));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_CANCEL;
     }
@@ -1046,7 +1093,9 @@ UUDecodeFile (uulist *thefile, char *destname)
 		 uustring (S_READ_ERROR),
 		 thefile->binfile, strerror (uu_errno = errno));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_IOERR;
     }
@@ -1056,20 +1105,25 @@ UUDecodeFile (uulist *thefile, char *destname)
 		 uustring (S_WR_ERR_TARGET),
 		 uugen_fnbuffer, strerror (uu_errno = errno));
       fclose (source);
+      UUCLRBUF (uu_rbuf, source_buf);
       fclose (target);
+      UUCLRBUF (uu_wbuf, target_buf);
       unlink (uugen_fnbuffer);
       return UURET_IOERR;
     }
   }
 
   fclose (source);
+  UUCLRBUF (uu_rbuf, source_buf);
   if (fclose (target)) {
+    UUCLRBUF (uu_wbuf, target_buf);
     UUMessage (uulib_id, __LINE__, UUMSG_ERROR,
 	       uustring (S_WR_ERR_TARGET),
 	       uugen_fnbuffer, strerror (uu_errno = errno));
     unlink (uugen_fnbuffer);
     return UURET_IOERR;
   }
+  UUCLRBUF (uu_wbuf, target_buf);
 
   /*
    * after a successful decoding run, we delete the temporary file
